/*
  test_resize_image.cxx

  Put `resize_image.cxx` in same folder and compile this source with
  `fltk-config --use-images --compile test_resize_image.cxx`

  Either supply an image file by command line or select one with
  the filechooser that comes up if none supplied.
  Scale the window to see different sizes.
*/

#include <FL/Fl_Window.H>
#include <FL/Fl_Box.H>
#include <FL/Fl_Shared_Image.H>
#include <FL/Fl_File_Chooser.H>
#include <FL/Fl.H>
#include <cstdio>
#include <iostream>
using namespace std;

#include "resize_image.cxx"	// the bilinear module fl_copy_image()

class Canvas : public Fl_Box
{
public:
	Canvas( int x, int y, int w, int h, const char *l = 0 ) :
		Fl_Box( x, y, w, h, l ),
		_image( 0 ),
		_resize_method( 0 )
	{
		align( FL_ALIGN_BOTTOM | FL_ALIGN_IMAGE_BACKDROP);
		box( FL_FLAT_BOX );
//		color( 0x22222200 );
		color( FL_YELLOW );
	}
	void set_image( const char *name, int resize_method = 0 )
	{
		if ( name )
			_image = Fl_Shared_Image::get( name );
		if ( _image )
			cout << name << " d=" << _image->d() << " count=" << _image->count() << endl;
		_resize_method = resize_method;
		resize_image();
	}
	void resize( int x, int y, int w, int h )
	{
		Fl_Box::resize( x, y, w, h );
		resize_image();
	}
private:
	void resize_image()
	{
		if ( !_image ) return;
		if ( !_image->w() || !_image->h() ) return;
		Fl_Image::RGB_scaling( FL_RGB_SCALING_BILINEAR );

		Fl_Image *temp_image = _image;
		if( _image->d() <= 1 || _image->count() != 1 )
		{
			temp_image = (Fl_Image *)(new Fl_RGB_Image( (Fl_Pixmap *)_image ) );
		}
		Fl_Image *img = _resize_method ?
			fl_copy_image( temp_image, w(), h() ) : temp_image->copy( w(), h() );
		delete image();
		image( img );
		if ( temp_image != _image )
			delete temp_image;
	}
	Fl_Image *_image;
	int _resize_method;
};

int main( int argc, const char *argv[] )
{
	const char *name = argc > 1 ? argv[1] : 0;
	fl_register_images();
	if ( !name )
		name = fl_file_chooser( "Select image:",
		"*.{bm,bmp,gif,jpg,pbm,pgm,png,ppm,xbm,xpm", NULL, 1 );

	Fl_Window win( 830, 330, name ? name : "test image scaling" );
	Canvas c1( 10, 10, 400, 300, "FLTK bilinear" );
	c1.set_image( name );
	Canvas c2( 420, 10, 400, 300, "RK bilinear" );
	c2.set_image( name, 1 );
	win.resizable( win );
	win.end();
	win.show();
	return Fl::run();
}
